# 交易审计

Orders 表

```postgresql
create table orders
(
    id          serial primary key,
    item_id     int,
    amount      int,
    unit_price  money,
    total       money,
    description text,
    ts          timestamp default now()
);
```

记录了未完成的订单。审计部门现在需要记录其变更——新增或删除,该表不会发生update——即将修改都记录到 orders_log 表

```postgresql
create table orders_log
(
    log_id      serial primary key,
    id          int,
    item_id     int,
    amount      int,
    unit_price  money,
    total       money,
    description text,
    ts          timestamp,
    direction   varchar(16),      
    log_at      timestamp default now()
);
```

那么应该如何做？

## 答案

```postgresql

create or replace function log_order_in() returns trigger as $$
begin
    insert into orders_log(id, item_id, amount, unit_price, total,  description, ts, direction)
    values(NEW.id, NEW.item_id, NEW.amount, NEW.unit_price, NEW.total, NEW.description, NEW.ts, 'in');
    return NEW;
end
$$ language plpgsql;

create or replace function log_order_out() returns trigger as $$
begin
    insert into orders_log(id, item_id, amount, unit_price, total,  description, ts, direction)
    values(OLD.id, OLD.item_id, OLD.amount, OLD.unit_price, OLD.total, OLD.description, OLD.ts, 'out');
    return OLD;
end
$$ language plpgsql;

create trigger in_orders after insert on orders
    for each row execute function log_order_in();

create trigger out_orders after delete on orders
    for each row execute function log_order_out();
```

## 选项

### A

```postgresql
create trigger in_orders after insert on orders
    for each row insert into orders_log(id, item_id, amount, unit_price, total, description, ts, direction)
    values(NEW.id, NEW.item_id, NEW.amount, NEW.unit_price, NEW.total, NEW.description, NEW.ts, 'in');

```

### B

```postgresql

create trigger out_orders after delete on orders
    for each row insert into orders_log(id, item_id, amount, unit_price, total, description, ts, direction)
                 values(OLD.id, OLD.item_id, OLD.amount, OLD.unit_price, OLD.total, OLD.description, OLD.ts, 'out');
```


### C

```postgresql
create trigger in_orders after insert on orders
        for each row insert into orders_log(id, item_id, amount, unit_price, total, description, ts, direction)
        values(NEW.id, NEW.item_id, NEW.amount, NEW.unit_price, NEW.total, NEW.description, NEW.ts, 'in');
    after delete on orders
        for each row insert into orders_log(id, item_id, amount, unit_price, total, description, ts, direction)
                     values(OLD.id, OLD.item_id, OLD.amount, OLD.unit_price, OLD.total, OLD.description, OLD.ts, 'out');
```

### D

```postgresql
create trigger in_orders after insert, after delete on orders
    for each row insert into orders_log(id, item_id, amount, unit_price, total, description, ts, direction)
    values(NEW.id, NEW.item_id, NEW.amount, NEW.unit_price, NEW.total, NEW.description, NEW.ts, 'in');
```